/*
		es.h
		Funciones de entrada/salida para el audio. Encapsula varias de las funcioens
		de sndlib para hacer ms facil el trabajo al abrir un archivo.

		Prefijos:
		Obt: para funciones que obtienen datos
		Asig: para funciones que asignan datos
		Impr: para funciones que imprimen datos en pantalla
*/



/* 
	chequea que la entrada de la lnea de comandos tenga el nmero de argumentos necesarios.
	Es una funcin general para todas las aplicaciones.

	args : la cantidad de argumentos que el usuario ingres.
	nargs : la cantidad argumentos necesarios para correr sta aplicacin.
	msj : el mensaje que se va a mostrar, dependiendo de la aplicacin.
	nomb_exe : nombre del ejecutable de sta aplicacin.
*/
int ChequearEntrada(int args, int nargs, char *msj, char *nomb_exe)
{
  if(args < nargs) 
		{
    fprintf(stderr, "\n  %s: Insuficientes argumentos [%d] \n  %s", nomb_exe, args, msj);
    fprintf(stdout, "\n\n Presione aceptar para terminar el programa ..."); 
    return -1;
		}
  return 0; 
}


/* 
	Obtiene los datos del audio  que se la pasa como nombre de archivo y los guarda en 'dau'
	Como se ve, la estructura se pasa en la lista de argumentos y no se devuelve porque ya 
	fu creada ntes, si no fuera as habria que crearla dentro y devolverla

	narch: nombre completo del archivo de audio, esto puede incluir la ruta.
	dau: la estructura que contendr todos los datos del encabezado del archivo de audio.

	Se usan las funciones de sndlib para leer distintos datos del encabezado de un audio.
*/
void ObtDatosAudio(char *narch, DAUDIO *dau)
{
	dau->frec_muestreo = mus_sound_srate(narch);
	dau->canales = mus_sound_chans(narch);
	dau->muestras = mus_sound_samples(narch);
	dau->mu_can  = mus_sound_frames(narch);
	dau->comentario = mus_sound_comment(narch); 
	dau->formato = mus_sound_data_format(narch);
	dau->cabecera = mus_sound_header_type(narch);
  dau->duracion = (double)dau->mu_can / (float)dau->frec_muestreo;
}


/* 
	Imprime en la consola informacin del archivo de audio guardados en 'dau'.
	Tambin se le pasa el nombre del archivo para obtener el tipo de la cabecera 
	y el formato del audio, stos datos no estn guardados en la estructura DAUDIO. 
	Por ltmo, se permite pasar un mensaje a ser mostrado junto con los datos como ttulo.

	msj: es utilizado como un ttulo que se agrega ntes del nombre del archvio de audio.
*/
void ImprDatosAudio(char *narch, DAUDIO *dau, char *titulo)
{
	fprintf(stdout, "\n%s: %s:\n\n  F.MUESTREO: \t%d  m/s\n  CANALES: \t%d\n  DURACION: \t%f  seg\n  MUESTRAS: \t%d\n  MUES/CAN: \t%d\n",
		titulo, narch, dau->frec_muestreo, dau->canales, dau->duracion, dau->muestras, dau->mu_can);

  fprintf(stdout, "  TIPO: \t%s\n  FORMATO: \t%s\n  COMENTARIO: \t%s\n", 
								  mus_header_type_name(mus_sound_header_type(narch)),
									mus_data_format_name(mus_sound_data_format(narch)),
									dau->comentario);
}


/* 
	Abre un archivo de audio para ser usado como entrada.
	La funcin realiza otras operaciones. Luego de la apertura del archivo,
	obtiene sus datos y los imprime en pantalla. Estas operaciones adicionales
	nos servirn como controles.

	Para eso, primero se debe abrir el archivo. Si todo sale bien, entonces
	obtener sus datos y luego imprimirlos en pantalla. La impresin de los datos en 
	pantalla no es necesaria para sta funcin pero nos asegura a nosotros que la operacin
	de escrbir los datos es correcta, ya que el nico control que la funcin hace es la 
	de si se pudo abrir el archivo.

	narch: nombre completo del archivo
	msj: el mensaje que es usado como ttulo al mostrar los datos en pantalla
*/
DAUDIO *AbrirAudioEntrada(char* narch, char* msj)
{
  DAUDIO *dau;                                   /* creamos un objeto de tipo DAUDIO */
  
	dau = (DAUDIO *)malloc(sizeof(DAUDIO));        /* le creamos memoria */
	dau->na = mus_sound_open_input(narch);         /* snd abre el archivo de audio, o lo crea si no existe */
  if (dau->na != -1)                             /* si no fall la apertura del archivo... */
    {
		ObtDatosAudio(narch, dau);                   /* obtenemos sus datos y lo cargamos en la estructura */
		ImprDatosAudio(narch, dau, msj);             /* imprimimos en pantalla los datos */
    }
  else                                           /* ...no se pudo abrir el archivo */
		{
    fprintf(stderr, "%s: %s\n", narch, strerror(errno));
    fprintf(stdout,"\n\n Presione aceptar para terminar el programa ..."); 
    getchar();
    return NULL;
		}
  return dau;                                    /* todo bien, retornamos la estructura cargada */
}



